@@ -20,13 +20,13 @@
 */
 #define MAKENSISW_CPP
 
-#include <windows.h>
+#include "makensisw.h"
 #include <windowsx.h>
 #include <stdio.h>
-#include "makensisw.h"
 #include "resource.h"
 #include "noclib.h"
 #include "toolbar.h"
+#include "update.h"
 
 NSCRIPTDATA g_sdata;
 NRESIZEDATA g_resize;
@@ -45,6 +45,7 @@ int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, char *cmdParam, int cmd
   g_sdata.hInstance=GetModuleHandle(0);
   g_sdata.script_alloced=false;
   g_sdata.symbols = NULL;
+  g_sdata.sigint_event = CreateEvent(NULL, FALSE, FALSE, "makensis win32 signint event");
   RestoreSymbols();
 
   if (!InitBranding()) {
@@ -70,6 +71,8 @@ int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hPrevInst, char *cmdParam, int cmd
     }
   }
   if (g_sdata.script_alloced) GlobalFree(g_sdata.script);
+  if (g_sdata.sigint_event) CloseHandle(g_sdata.sigint_event);
+  FinalizeUpdate();
   ExitProcess(msg.wParam);
   return msg.wParam;
 }
@@ -103,9 +106,9 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam) {
       SendMessage(GetDlgItem(hwndDlg,IDC_LOGWIN),EM_SETEVENTMASK,NULL,ENM_SELCHANGE|ENM_MOUSEEVENTS|ENM_KEYEVENTS);
       DragAcceptFiles(g_sdata.hwnd,FALSE);
       g_sdata.menu = GetMenu(g_sdata.hwnd);
-      g_sdata.fileSubmenu = GetSubMenu(g_sdata.menu, FILE_MENU_INDEX);
-      g_sdata.editSubmenu = GetSubMenu(g_sdata.menu, EDIT_MENU_INDEX);
-      g_sdata.toolsSubmenu = GetSubMenu(g_sdata.menu, TOOLS_MENU_INDEX);
+      g_sdata.fileSubmenu = FindSubMenu(g_sdata.menu, IDM_FILE);
+      g_sdata.editSubmenu = FindSubMenu(g_sdata.menu, IDM_EDIT);
+      g_sdata.toolsSubmenu = FindSubMenu(g_sdata.menu, IDM_TOOLS);
       RestoreMRUList();
       CreateToolBar();
       InitTooltips(g_sdata.hwnd);
@@ -193,9 +196,9 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam) {
     {
       if (!g_sdata.thread) {
         DragAcceptFiles(g_sdata.hwnd,FALSE);
-		ImageList_Destroy(g_toolbar.imagelist);
-		ImageList_Destroy(g_toolbar.imagelistd);
-		ImageList_Destroy(g_toolbar.imagelisth);
+        ImageList_Destroy(g_toolbar.imagelist);
+        ImageList_Destroy(g_toolbar.imagelistd);
+        ImageList_Destroy(g_toolbar.imagelisth);
         DestroyWindow(hwndDlg);
         FreeLibrary(hRichEditDLL);
       }
@@ -215,7 +218,10 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam) {
           ResetObjects();
           CompileNSISScript();
         }
+      } else {
+        MessageBox(hwndDlg,MULTIDROPERROR,"Error",MB_OK|MB_ICONSTOP);
       }
+      DragFinish((HDROP)wParam);
       break;
     }
     case WM_GETMINMAXINFO:
@@ -465,17 +471,7 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam) {
         }
         case IDM_NSISUPDATE:
         {
-          int rv;
-          rv = MessageBox(g_sdata.hwnd,NSISUPDATEPROMPT,"Question",MB_YESNO|MB_ICONQUESTION);
-          if(rv == IDYES) {
-            char pathf[MAX_PATH],*path;
-            GetModuleFileName(NULL,pathf,sizeof(pathf));
-            path=my_strrchr(pathf,'\\');
-            if(path!=NULL) *path=0;
-            lstrcat(pathf,NSIS_UPDATE);
-            ShellExecute(g_sdata.hwnd,"open",pathf,NULL,NULL,SW_SHOWNORMAL);
-            PostMessage(g_sdata.hwnd,WM_COMMAND, IDM_EXIT, 0);
-          }
+          Update();
           return TRUE;
         }
         case IDM_SELECTALL:
@@ -582,6 +578,11 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lParam) {
           }
           return TRUE;
         }
+        case IDM_CANCEL:
+        {
+          SetEvent(g_sdata.sigint_event);
+          return TRUE;
+        }
         case IDM_COPY:
         {
           CopyToClipboard(g_sdata.hwnd);
@@ -675,6 +676,7 @@ DWORD WINAPI MakeNSISProc(LPVOID p) {
   SECURITY_DESCRIPTOR sd={0,};
   PROCESS_INFORMATION pi={0,};
   HANDLE newstdout=0,read_stdout=0;
+  HANDLE newstdin=0,read_stdin=0;
   OSVERSIONINFO osv={sizeof(osv)};
   GetVersionEx(&osv);
   if (osv.dwPlatformId == VER_PLATFORM_WIN32_NT) {
@@ -685,7 +687,12 @@ DWORD WINAPI MakeNSISProc(LPVOID p) {
   else sa.lpSecurityDescriptor = NULL;
   sa.bInheritHandle = true;
   if (!CreatePipe(&read_stdout,&newstdout,&sa,0)) {
-    ErrorMessage(g_sdata.hwnd,"There was an error creating the pipe.");
+    ErrorMessage(g_sdata.hwnd,"There was an error creating the output pipe.");
+    PostMessage(g_sdata.hwnd,WM_MAKENSIS_PROCESSCOMPLETE,0,0);
+    return 1;
+  }
+  if (!CreatePipe(&read_stdin,&newstdin,&sa,0)) {
+    ErrorMessage(g_sdata.hwnd,"There was an error creating the input pipe.");
     PostMessage(g_sdata.hwnd,WM_MAKENSIS_PROCESSCOMPLETE,0,0);
     return 1;
   }
@@ -694,6 +701,7 @@ DWORD WINAPI MakeNSISProc(LPVOID p) {
   si.wShowWindow = SW_HIDE;
   si.hStdOutput = newstdout;
   si.hStdError = newstdout;
+  si.hStdInput = newstdin;
   if (!CreateProcess(NULL,g_sdata.script,NULL,NULL,TRUE,CREATE_NEW_CONSOLE,NULL,NULL,&si,&pi)) {
     char buf[MAX_STRING];
     wsprintf(buf,"Could not execute:\r\n %s.",g_sdata.script);
@@ -726,6 +734,8 @@ DWORD WINAPI MakeNSISProc(LPVOID p) {
   CloseHandle(pi.hProcess);
   CloseHandle(newstdout);
   CloseHandle(read_stdout);
+  CloseHandle(newstdin);
+  CloseHandle(read_stdin);
   PostMessage(g_sdata.hwnd,WM_MAKENSIS_PROCESSCOMPLETE,0,0);
   return 0;
 }
@@ -1097,7 +1107,7 @@ BOOL CALLBACK CompressorProc(HWND hwndDlg, UINT msg, WPARAM wParam, LPARAM lPara
         case IDCANCEL:
         {
           EndDialog(hwndDlg, 1);
-		  LogMessage(g_sdata.hwnd,USAGE);
+          LogMessage(g_sdata.hwnd,USAGE);
           break;
         }
       }
@@ -1243,7 +1253,7 @@ void SetCompressor(NCOMPRESSOR compressor)
     }
     else {
       compressor = COMPRESSOR_SCRIPT;
-      command = IDM_SCRIPT;
+      command = IDM_COMPRESSOR_SCRIPT;
       compressor_name = "";
     }
     g_sdata.compressor = compressor;
