@@ -6,9 +6,6 @@
 // includes
 
 #ifdef _WIN32
-#  ifndef _WIN32_IE
-#    define _WIN32_IE 0x0400
-#  endif
 #  include <windows.h>
 #  include <commctrl.h>
 #else
@@ -40,7 +37,7 @@ typedef unsigned short WCHAR, *PWCHAR, *LPWCH, *PWCH, *NWPSTR, *LPWSTR, *PWSTR;
 typedef const unsigned short *LPCWCH, *PCWCH, *LPCWSTR, *PCWSTR;
 typedef unsigned int UINT_PTR;
 // basic stuff
-typedef unsigned long HANDLE;
+typedef void * HANDLE;
 typedef unsigned long HKEY;
 // some gdi
 typedef unsigned long COLORREF;
@@ -52,6 +49,25 @@ typedef unsigned long HBRUSH;
 typedef WORD LANGID;
 #endif
 
+#ifndef __BIG_ENDIAN__
+# define FIX_ENDIAN_INT32_INPLACE(x) (x)
+# define FIX_ENDIAN_INT32(x) (x)
+# define FIX_ENDIAN_INT16_INPLACE(x) (x)
+# define FIX_ENDIAN_INT16(x) (x)
+#else
+# define FIX_ENDIAN_INT32_INPLACE(x) ((x) = SWAP_ENDIAN_INT32(x))
+# define FIX_ENDIAN_INT32(x) SWAP_ENDIAN_INT32(x)
+# define FIX_ENDIAN_INT16_INPLACE(x) ((x) = SWAP_ENDIAN_INT16(x))
+# define FIX_ENDIAN_INT16(x) SWAP_ENDIAN_INT16(x)
+#endif
+#define SWAP_ENDIAN_INT32(x) ( \
+  (((x)&0xFF000000) >> 24) | \
+  (((x)&0x00FF0000) >>  8) | \
+  (((x)&0x0000FF00) <<  8) | \
+  (((x)&0x000000FF) << 24) )
+#define SWAP_ENDIAN_INT16(x) ( \
+  (((x)&0xFF00) >> 8) | \
+  (((x)&0x00FF) << 8) )
 
 // script path separator
 
@@ -112,12 +128,6 @@ typedef WORD LANGID;
 // macros
 
 #ifndef _WIN32
-#  ifndef min
-#    define min(x,y) ((x<y)?x:y)
-#  endif
-#  ifndef max
-#    define max(x,y) ((x>y)?x:y)
-#  endif
 #  ifndef FIELD_OFFSET
 #    define FIELD_OFFSET(t,f) ((LONG)&(((t*)0)->f))
 #  endif
@@ -125,7 +135,9 @@ typedef WORD LANGID;
 #    define MAKEINTRESOURCE(i) (LPSTR)((DWORD)((WORD)(i)))
 #  endif
 #  ifndef IMAGE_FIRST_SECTION
-#    define IMAGE_FIRST_SECTION(h) ((PIMAGE_SECTION_HEADER) ((DWORD)h+FIELD_OFFSET(IMAGE_NT_HEADERS,OptionalHeader)+((PIMAGE_NT_HEADERS)(h))->FileHeader.SizeOfOptionalHeader))
+#    define IMAGE_FIRST_SECTION(h) ( PIMAGE_SECTION_HEADER( (DWORD) h + \
+                                     FIELD_OFFSET(IMAGE_NT_HEADERS, OptionalHeader) + \
+                                     FIX_ENDIAN_INT16(PIMAGE_NT_HEADERS(h)->FileHeader.SizeOfOptionalHeader) ) )
 #  endif
 #  ifndef RGB
 #    define RGB(r,g,b) ((DWORD)(((BYTE)(r)|((WORD)(g)<<8))|(((DWORD)(BYTE)(b))<<16)))
@@ -271,6 +283,9 @@ typedef WORD LANGID;
 #ifndef CSIDL_COMMON_APPDATA
 #  define CSIDL_COMMON_APPDATA 0x23
 #endif
+#ifndef CSIDL_LOCAL_APPDATA
+#  define CSIDL_LOCAL_APPDATA 0x1C
+#endif
 #ifndef CSIDL_PRINTHOOD
 #  define CSIDL_PRINTHOOD 0x1B
 #endif
@@ -338,6 +353,14 @@ typedef WORD LANGID;
 #  define CSIDL_CDBURN_AREA 0x3B
 #endif
 
+// other shell stuff
+
+#ifndef SHACF_FILESYSTEM
+#  define SHACF_FILESYSTEM 1
+#endif
+
+// other stuff
+
 #ifndef CP_ACP
 #  define CP_ACP 0
 #endif
@@ -349,6 +372,8 @@ typedef WORD LANGID;
 #  define COLOR_WINDOW 5
 #endif
 
+// resources
+
 #ifndef RT_BITMAP
 #  define RT_BITMAP MAKEINTRESOURCE(2)
 #endif
@@ -365,6 +390,8 @@ typedef WORD LANGID;
 #  define RT_VERSION MAKEINTRESOURCE(16)
 #endif
 
+// version
+
 #ifndef VS_FILE_INFO
 #  define VS_FILE_INFO RT_VERSION
 #endif
@@ -442,6 +469,7 @@ typedef WORD LANGID;
 #  define WS_EX_RIGHT 0x1000
 #  define WS_EX_RIGHTSCROLLBAR 0
 #  define WS_EX_RTLREADING 0x2000
+#  define WS_EX_LEFTSCROLLBAR 0x4000
 
 #  define TVS_RTLREADING 64
 
@@ -587,8 +615,13 @@ typedef WORD LANGID;
 
 #ifndef _WIN32
 #  define IMAGE_NUMBEROF_DIRECTORY_ENTRIES 16
-#  define IMAGE_DOS_SIGNATURE 0x5A4D
-#  define IMAGE_NT_SIGNATURE 0x00004550
+#  ifndef __BIG_ENDIAN__
+#    define IMAGE_DOS_SIGNATURE 0x5A4D
+#    define IMAGE_NT_SIGNATURE 0x00004550
+#  else
+#    define IMAGE_DOS_SIGNATURE 0x4D5A
+#    define IMAGE_NT_SIGNATURE 0x50450000
+#  endif
 #  define IMAGE_FILE_DLL 8192
 #  define IMAGE_DIRECTORY_ENTRY_EXPORT 0
 #  define IMAGE_SIZEOF_SHORT_NAME 8
@@ -635,6 +668,7 @@ typedef struct _IMAGE_DOS_HEADER {
   WORD e_res2[10];
   LONG e_lfanew;
 } IMAGE_DOS_HEADER,*PIMAGE_DOS_HEADER;
+#  pragma pack()
 #  pragma pack(4)
 typedef struct _IMAGE_FILE_HEADER {
   WORD Machine;
