@@ -1,5 +1,5 @@
 /*
-*  Copyright (C) 1999-2005 Nullsoft, Inc.
+*  Copyright (C) 1999-2006 Nullsoft, Inc.
 *  Portions Copyright (C) 2002 Jeff Doozan
 *
 *  This software is provided 'as-is', without any express or implied warranty.
@@ -20,10 +20,12 @@
 *  3. This notice may not be removed or altered from any source distribution.
 */
 
-#include "../Platform.h"
 #include <windowsx.h>
 #include <shlobj.h>
 #include <shellapi.h>
+#include <shlwapi.h>
+
+#include "../Platform.h"
 
 #include "resource.h"
 
@@ -49,7 +51,11 @@ int g_quit_flag; // set when Quit has been called (meaning bail out ASAP)
 
 int progress_bar_pos, progress_bar_len;
 
+#if NSIS_MAX_STRLEN < 1024
 static char g_tmp[4096];
+#else
+static char g_tmp[NSIS_MAX_STRLEN * 4];
+#endif
 
 static int m_page=-1,m_retcode,m_delta;
 static page *g_this_page;
@@ -159,10 +165,10 @@ static BOOL NSISCALL _HandleStaticBkColor(UINT uMsg, WPARAM wParam, LPARAM lPara
 #endif//!NSIS_CONFIG_ENHANCEDUI_SUPPORT
 
 #ifdef NSIS_CONFIG_LOG
-#ifndef NSIS_CONFIG_LOG_ODS
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
 void NSISCALL build_g_logfile()
 {
-  lstrcat(addtrailingslash(mystrcpy(g_log_file,state_install_directory)),"install.log");
+  mystrcat(addtrailingslash(mystrcpy(g_log_file,state_install_directory)),"install.log");
 }
 #endif
 #endif
@@ -172,9 +178,10 @@ int *cur_langtable;
 static void NSISCALL set_language()
 {
   LANGID lang_mask=(LANGID)~0;
-  LANGID lang=state_language[0]?myatoi(state_language):GetUserDefaultLangID();
+  LANGID lang=myatoi(state_language);
   char *language_table=0;
   int lang_num;
+  int *selected_langtable=0;
 
 lang_again:
   lang_num=g_blocks[NB_LANGTABLES].num;
@@ -183,11 +190,11 @@ lang_again:
     if (!((lang ^ *(LANGID*)language_table) & lang_mask)) {
       dlg_offset=*(int*)(language_table+sizeof(LANGID));
       g_exec_flags.rtl=*(int*)(language_table+sizeof(LANGID)+sizeof(int));
-      cur_langtable=(int*)(language_table+sizeof(LANGID)+2*sizeof(int));
+      selected_langtable=(int*)(language_table+sizeof(LANGID)+2*sizeof(int));
       break;
     }
   }
-  if (!cur_langtable) {
+  if (!selected_langtable) {
     if (lang_mask == (LANGID)~0)
       lang_mask=0x3ff; // primary lang
     else // we already tried once and we still don't have a language table
@@ -195,6 +202,8 @@ lang_again:
     goto lang_again;
   }
 
+  cur_langtable = selected_langtable;
+
   myitoa(state_language, *(LANGID*)language_table);
   {
     char *caption = GetNSISString(g_caption,LANG_CAPTION);
@@ -223,10 +232,51 @@ FORCE_INLINE int NSISCALL ui_doinstall(void)
 {
   header *header = g_header;
   static WNDCLASS wc; // richedit subclassing and bgbg creation
-  g_exec_flags.autoclose=g_flags&CH_FLAGS_AUTO_CLOSE;
 
+  // detect default language
+  // more information at:
+  //   http://msdn.microsoft.com/library/default.asp?url=/library/en-us/intl/nls_0xrn.asp
+
+  LANGID (WINAPI *GUDUIL)();
+  static const char guduil[] = "GetUserDefaultUILanguage";
+
+  GUDUIL = myGetProcAddress("KERNEL32.dll", (char *) guduil);
+  if (GUDUIL)
+  {
+    // Windows ME/2000+
+    myitoa(state_language, GUDUIL());
+  }
+  else
+  {
+    *(DWORD*)state_language = CHAR4_TO_DWORD('0', 'x', 0, 0);
+
+    {
+      // Windows 9x
+      static const char reg_9x_locale[] = "Control Panel\\Desktop\\ResourceLocale";
+
+      myRegGetStr(HKEY_CURRENT_USER, reg_9x_locale, NULL, g_tmp);
+    }
+
+    if (!g_tmp[0])
+    {
+      // Windows NT
+      // This key exists on 9x as well, so it's only read if ResourceLocale wasn't found
+      static const char reg_nt_locale_key[] = ".DEFAULT\\Control Panel\\International";
+      static const char reg_nt_locale_val[] = "Locale";
+
+      myRegGetStr(HKEY_USERS, reg_nt_locale_key, reg_nt_locale_val, g_tmp);
+    }
+
+    mystrcat(state_language, g_tmp);
+  }
+
+  // set default language
   set_language();
 
+  // initialize auto close flag
+  g_exec_flags.autoclose=g_flags&CH_FLAGS_AUTO_CLOSE;
+
+  // read install directory from registry
   if (!is_valid_instpath(state_install_directory))
   {
     if (header->install_reg_key_ptr)
@@ -279,7 +329,7 @@ FORCE_INLINE int NSISCALL ui_doinstall(void)
 #ifdef NSIS_CONFIG_LOG
   if (g_flags & CH_FLAGS_SILENT_LOG && !g_is_uninstaller)
   {
-#ifndef NSIS_CONFIG_LOG_ODS
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
     build_g_logfile();
 #endif
     log_dolog=1;
@@ -433,6 +483,8 @@ BOOL CALLBACK DialogProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
       m_hwndCancel=GetDlgItem(hwndDlg,IDCANCEL);
       SetDlgItemTextFromLang(hwndDlg,IDC_VERSTR,LANG_BRANDING);
       SetClassLong(hwndDlg,GCL_HICON,(long)g_hIcon);
+      // use the following line instead of the above, if .rdata needs shirking
+      //SendMessage(hwndDlg,WM_SETICON,ICON_BIG,(LPARAM)g_hIcon);
 #if defined(NSIS_SUPPORT_CODECALLBACKS) && defined(NSIS_CONFIG_ENHANCEDUI_SUPPORT)
       g_quit_flag = ExecuteCallbackFunction(CB_ONGUIINIT);
 #endif
@@ -726,14 +778,11 @@ static BOOL CALLBACK LicenseProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM
         };
         if (tr.chrg.cpMax-tr.chrg.cpMin < sizeof(ps_tmpbuf)) {
           SendMessage(hwLicense,EM_GETTEXTRANGE,0,(LPARAM)&tr);
-          SetCursor(LoadCursor(0,IDC_WAIT));
+          SetCursor(LoadCursor(0, IDC_WAIT));
           ShellExecute(hwndDlg,"open",tr.lpstrText,NULL,NULL,SW_SHOWNORMAL);
-          SetCursor(LoadCursor(0,IDC_ARROW));
+          SetCursor(LoadCursor(0, IDC_ARROW));
         }
       }
-      if (enlink->msg==WM_SETCURSOR) {
-        SetCursor(LoadCursor(0,IDC_HAND));
-      }
     }
     //Ximon Eighteen 8th September 2002 Capture return key presses in the rich
     //edit control now that the control gets the focus rather than the default
@@ -781,19 +830,24 @@ static BOOL CALLBACK UninstProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM l
 static void NSISCALL SetSizeText(int dlgItem, int prefix, unsigned kb)
 {
   char scalestr[32], byte[32];
-  unsigned sh=20;
-  int scale=LANG_GIGA;
-  
-  if (kb < 1024) { sh=0; scale=LANG_KILO; }
-  else if (kb < 1024*1024) { sh=10; scale=LANG_MEGA; }
+  unsigned sh = 20;
+  int scale = LANG_GIGA;
+
+  if (kb < 1024 * 1024) { sh = 10; scale = LANG_MEGA; }
+  if (kb < 1024) { sh = 0; scale = LANG_KILO; }
+
+  if (kb < (0xFFFFFFFF - ((1 << 20) / 20))) // check for overflow
+    kb += (1 << sh) / 20; // round numbers for better display (e.g. 1.59 => 1.6)
 
   wsprintf(
-    GetNSISString(g_tmp,prefix)+mystrlen(g_tmp),
+    GetNSISString(g_tmp, prefix) + mystrlen(g_tmp),
     "%u.%u%s%s",
-    kb>>sh,
-    ((kb*10)>>sh)%10,
-    GetNSISString(scalestr,scale),
-    GetNSISString(byte,LANG_BYTE)
+    kb >> sh,
+    (((kb & 0x00FFFFFF) * 10) >> sh) % 10, // 0x00FFFFFF mask is used to
+                                           // prevent overflow that causes
+                                           // bad results
+    GetNSISString(scalestr, scale),
+    GetNSISString(byte, LANG_BYTE)
   );
 
   my_SetDialogItemText(m_curwnd,dlgItem,g_tmp);
@@ -828,14 +882,17 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     GetUIText(IDC_DIR,dir);
     validate_filename(dir);
 #ifdef NSIS_CONFIG_LOG
-#ifndef NSIS_CONFIG_LOG_ODS
+#if !defined(NSIS_CONFIG_LOG_ODS) && !defined(NSIS_CONFIG_LOG_STDOUT)
     build_g_logfile();
 #endif
-    log_dolog = IsDlgButtonChecked(hwndDlg,IDC_CHECK1);
+    if (GetUIItem(IDC_CHECK1) != NULL)
+      log_dolog = IsDlgButtonChecked(hwndDlg,IDC_CHECK1);
 #endif
   }
   if (uMsg == WM_INITDIALOG)
   {
+    HWND hDir = GetUIItem(IDC_DIR);
+
 #ifdef NSIS_CONFIG_LOG
     if (GetAsyncKeyState(VK_SHIFT)&0x8000)
     {
@@ -846,10 +903,38 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
 #endif
     if (validpathspec(dir) && !skip_root(dir))
       addtrailingslash(dir);
-    SetUITextNT(IDC_DIR,dir);
+
+    // workaround for bug #1209843
+    //
+    // m_curwnd is only updated once WM_INITDIALOG returns.
+    // my_SetWindowText triggers an EN_CHANGE message that
+    // triggers a WM_IN_UPDATEMSG message that uses m_curwnd
+    // to get the selected directory (GetUIText).
+    // because m_curwnd is still outdated, dir varialble is
+    // filled with an empty string. by default, dir points
+    // to $INSTDIR.
+    //
+    // to solve this, m_curwnd is manually set to the correct
+    // window handle.
+
+    m_curwnd=hwndDlg;
+
+    my_SetWindowText(hDir,dir);
     SetUITextFromLang(IDC_BROWSE,this_page->parms[2]);
     SetUITextFromLang(IDC_SELDIRTEXT,this_page->parms[1]);
-    SetActiveCtl(GetUIItem(IDC_DIR));
+    SetActiveCtl(hDir);
+
+    {
+      typedef HRESULT (WINAPI *SHAutoCompletePtr)(HWND, DWORD);
+      SHAutoCompletePtr fSHAutoComplete;
+      static const char shlwapi[] = "shlwapi.dll";
+      static const char shac[] = "SHAutoComplete";
+      fSHAutoComplete = (SHAutoCompletePtr) myGetProcAddress((char *) shlwapi, (char *) shac);
+      if (fSHAutoComplete)
+      {
+        fSHAutoComplete(hDir, SHACF_FILESYSTEM);
+      }
+    }
   }
   if (uMsg == WM_COMMAND)
   {
@@ -874,6 +959,8 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
         // free idlist
         FreePIDL(idlist);
 
+        addtrailingslash(dir);
+
         if (g_header->install_directory_auto_append)
         {
           const char *post_str = ps_tmpbuf;
@@ -881,13 +968,17 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
           // display name gives just the folder name
           if (lstrcmpi(post_str, g_tmp))
           {
-            lstrcat(addtrailingslash(dir), post_str);
+            mystrcat(dir, post_str);
           }
         }
 
         dontsetdefstyle++;
         SetUITextNT(IDC_DIR,dir);
       }
+      else
+      {
+        uMsg = WM_IN_UPDATEMSG;
+      }
     }
   }
   if (uMsg == WM_IN_UPDATEMSG || uMsg == WM_NOTIFY_START)
@@ -897,7 +988,6 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     int error = 0;
     int available_set = 0;
     unsigned total, available = 0xFFFFFFFF;
-    HMODULE hLib;
 
     GetUIText(IDC_DIR,dir);
     if (!is_valid_instpath(dir))
@@ -909,22 +999,23 @@ static BOOL CALLBACK DirProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
       *p=0;
 
     // Test for and use the GetDiskFreeSpaceEx API
-    hLib = GetModuleHandle("KERNEL32.dll");
-    if (hLib)
     {
       BOOL (WINAPI *GDFSE)(LPCSTR, PULARGE_INTEGER, PULARGE_INTEGER, PULARGE_INTEGER) =
-        (void*)GetProcAddress(hLib, "GetDiskFreeSpaceExA");
+          myGetProcAddress("KERNEL32.dll", "GetDiskFreeSpaceExA");
       if (GDFSE)
       {
         ULARGE_INTEGER available64;
         ULARGE_INTEGER a, b;
         if (GDFSE(s, &available64, &a, &b))
         {
+#ifndef _NSIS_NO_INT64_SHR
           available = (int)(available64.QuadPart >> 10);
+#else
+          available = (int)(Int64ShrlMod32(available64.QuadPart, 10));
+#endif
           available_set++;
         }
       }
-
     }
 
     if (!available_set)
@@ -991,8 +1082,6 @@ static void FORCE_INLINE NSISCALL RefreshComponents(HWND hwTree, HTREEITEM *item
     item.hItem = items[i];
 
     item.mask = TVIF_STATE;
-    item.state = (flags & SF_BOLD) << 1; // (SF_BOLD << 1) == 16 == TVIS_BOLD
-    item.state |= flags & SF_EXPAND; // TVIS_EXPANDED == SF_EXPAND
 
     if (flags & SF_NAMECHG)
     {
@@ -1011,32 +1100,61 @@ static void FORCE_INLINE NSISCALL RefreshComponents(HWND hwTree, HTREEITEM *item
       if (flags & SF_RO) state += 3;
     }
 
+    item.state = (flags & SF_BOLD) << 1; // (SF_BOLD << 1) == 16 == TVIS_BOLD
+    item.state |= flags & SF_EXPAND; // TVIS_EXPANDED == SF_EXPAND
     item.state |= INDEXTOSTATEIMAGEMASK(state);
 
-    TreeView_Expand(hwTree, item.hItem, (flags & SF_EXPAND) ? TVE_EXPAND : TVE_COLLAPSE);
+    // TVE_COLLAPSE = 1, TVE_EXPAND = 2
+    TreeView_Expand(hwTree, item.hItem, TVE_COLLAPSE + ((flags & SF_EXPAND) / SF_EXPAND));
+
     TreeView_SetItem(hwTree, &item);
   }
+
+  // workaround for bug #1397031
+  //
+  // windows 95 doesn't erase the background of the state image
+  // before it draws a new one. because of this parts of the old
+  // state image will show where the new state image is masked.
+  //
+  // to solve this, the following line forces the background to
+  // be erased. sadly, this redraws the entire control. it might
+  // be a good idea to figure out where the state images are and
+  // redraw only those.
+
+  InvalidateRect(hwTree, NULL, TRUE);
 }
 
-HTREEITEM NSISCALL TreeHitTest(HWND tree)
+int NSISCALL TreeGetSelectedSection(HWND tree, BOOL mouse)
 {
-  TVHITTESTINFO ht;
-  DWORD dwpos = GetMessagePos();
+  HTREEITEM hItem = TreeView_GetSelection(tree);
+  TVITEM item;
 
-  ht.pt.x = GET_X_LPARAM(dwpos);
-  ht.pt.y = GET_Y_LPARAM(dwpos);
-  ScreenToClient(tree, &ht.pt);
+  if (mouse)
+  {
+    TVHITTESTINFO ht;
+    DWORD dwpos = GetMessagePos();
 
-  TreeView_HitTest(tree, &ht);
+    ht.pt.x = GET_X_LPARAM(dwpos);
+    ht.pt.y = GET_Y_LPARAM(dwpos);
+    ScreenToClient(tree, &ht.pt);
+
+    TreeView_HitTest(tree, &ht);
 
 #ifdef NSIS_CONFIG_COMPONENTPAGE_ALTERNATIVE
-  if (ht.flags & TVHT_ONITEMSTATEICON)
+    if (!(ht.flags & TVHT_ONITEMSTATEICON))
 #else
-  if (ht.flags & (TVHT_ONITEMSTATEICON|TVHT_ONITEMLABEL|TVHT_ONITEMRIGHT|TVHT_ONITEM))
+    if (!(ht.flags & (TVHT_ONITEMSTATEICON|TVHT_ONITEMLABEL|TVHT_ONITEMRIGHT|TVHT_ONITEM)))
 #endif
-    return ht.hItem;
+      return -1;
 
-  return 0;
+    hItem = ht.hItem;
+  }
+
+  item.mask = TVIF_PARAM;
+  item.hItem = hItem;
+  TreeView_GetItem(tree, &item);
+
+  return (int) item.lParam;
 }
 
 static LONG oldTreeWndProc;
@@ -1053,21 +1171,8 @@ static DWORD WINAPI newTreeWndProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM l
   }
 #ifndef NSIS_CONFIG_COMPONENTPAGE_ALTERNATIVE
   if (uMsg == WM_MOUSEMOVE) {
-    TVITEM tvItem;
-
     if (IsWindowVisible(hwnd)) {
-      tvItem.hItem = TreeHitTest(hwnd);
-
-      lParam = -1;
-
-      if (tvItem.hItem)
-      {
-        tvItem.mask = TVIF_PARAM;
-
-        TreeView_GetItem(hwnd, &tvItem);
-
-        lParam = tvItem.lParam;
-      }
+      lParam = TreeGetSelectedSection(hwnd, TRUE);
       uMsg = WM_NOTIFY_SELCHANGE;
     }
   }
@@ -1108,7 +1213,7 @@ static BOOL CALLBACK SelProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
 
     g_SectionHack=hwndDlg;
 
-    hTreeItems=(HTREEITEM*)my_GlobalAlloc(sizeof(HTREEITEM)*num_sections);
+    hTreeItems=(HTREEITEM*)GlobalAlloc(GPTR,sizeof(HTREEITEM)*num_sections);
 
     hBMcheck1=LoadBitmap(g_hInstance, MAKEINTRESOURCE(IDB_BITMAP1));
 
@@ -1119,8 +1224,8 @@ static BOOL CALLBACK SelProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
 
     TreeView_SetImageList(hwndTree1, hImageList, TVSIL_STATE);
 
-    if (SendMessage(hwndTree1, TVM_GETITEMHEIGHT, 0, 0) < 16)
-      SendMessage(hwndTree1, TVM_SETITEMHEIGHT, 16, 0);
+    if (TreeView_GetItemHeight(hwndTree1) < 16)
+      TreeView_SetItemHeight(hwndTree1, 16);
 
     DeleteObject(hBMcheck1);
 
@@ -1180,7 +1285,6 @@ static BOOL CALLBACK SelProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     {
       SetWindowLong(hwndTree1,GWL_STYLE,GetWindowLong(hwndTree1,GWL_STYLE)&~(TVS_LINESATROOT));
     }
-    SendMessage(hwndTree1,WM_VSCROLL,SB_TOP,0);
 
     if (!noCombo)
     {
@@ -1205,48 +1309,39 @@ static BOOL CALLBACK SelProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPar
     {
       if (!(g_flags&CH_FLAGS_NO_CUSTOM) && (uMsg == WM_TREEVIEW_KEYHACK || lpnmh->code == NM_CLICK))
       {
-        TVITEM tvItem;
+        int secid = TreeGetSelectedSection(hwndTree1, uMsg != WM_TREEVIEW_KEYHACK);
 
-        if (uMsg != WM_TREEVIEW_KEYHACK)
-          tvItem.hItem=TreeHitTest(hwndTree1);
-        else
-          tvItem.hItem=TreeView_GetSelection(hwndTree1);
-
-        if (tvItem.hItem)
+        if (secid >= 0)
         {
-          tvItem.mask = TVIF_PARAM;
-          if (TreeView_GetItem(hwndTree1, &tvItem))
+          int flags = sections[secid].flags;
+
+          if ((flags & SF_RO) == 0)
           {
-            int flags = sections[tvItem.lParam].flags;
-
-            if ((flags & SF_RO) == 0)
+            if ((flags & SF_PSELECTED))
             {
-              if ((flags & SF_PSELECTED))
-              {
-                flags ^= SF_TOGGLED;
+              flags ^= SF_TOGGLED;
 
-                if (flags & SF_TOGGLED)
-                {
-                  flags |= SF_SELECTED;
-                }
-                else
-                {
-                  flags &= ~SF_SELECTED;
-                }
+              if (flags & SF_TOGGLED)
+              {
+                flags |= SF_SELECTED;
               }
               else
               {
-                flags ^= SF_SELECTED;
+                flags &= ~SF_SELECTED;
               }
-
-              sections[tvItem.lParam].flags = flags;
-
-              SectionFlagsChanged(tvItem.lParam);
-
-              wParam = 1;
-              lParam = !(g_flags & CH_FLAGS_COMP_ONLY_ON_CUSTOM);
-              uMsg = WM_IN_UPDATEMSG;
             }
+            else
+            {
+              flags ^= SF_SELECTED;
+            }
+
+            sections[secid].flags = flags;
+
+            SectionFlagsChanged(secid);
+
+            wParam = 1;
+            lParam = !(g_flags & CH_FLAGS_COMP_ONLY_ON_CUSTOM);
+            uMsg = WM_IN_UPDATEMSG;
           }
         } // was valid click
       } // was click or hack
@@ -1378,7 +1473,7 @@ void NSISCALL update_status_text(int strtab, const char *text) {
     if (text)
     {
       if (tmplen + mystrlen(text) >= sizeof(tmp)) return;
-      lstrcat(tmp, text);
+      mystrcat(tmp, text);
     }
 
     if ((updateflag & 4)) my_SetWindowText(insthwnd2, tmp);
@@ -1388,7 +1483,8 @@ void NSISCALL update_status_text(int strtab, const char *text) {
       new_item.pszText = tmp;
       new_item.iItem = ListView_GetItemCount(linsthwnd) - (updateflag & 1);
       new_item.iSubItem = 0;
-      SendMessage(linsthwnd, (updateflag & 1) ? LVM_SETITEM : LVM_INSERTITEM, 0, (LPARAM) &new_item);
+      // LVM_INSERTITEM - LVM_SETITEM = 1
+      SendMessage(linsthwnd, LVM_INSERTITEM - (updateflag & 1), 0, (LPARAM) &new_item);
       ListView_EnsureVisible(linsthwnd, new_item.iItem, 0);
     }
 
@@ -1410,6 +1506,17 @@ static DWORD WINAPI install_thread(LPVOID p)
   }
 #endif
 
+  // workaround for bug #1400995
+  //
+  // for an unexplained reason, MessageBox with MB_TOPMOST
+  // will fail, if no other messages were sent from this
+  // thread to the GUI thread before it.
+  //
+  // the source of the problem couldn't be found, so a
+  // WM_NULL is sent to work around it.
+
+  NotifyCurWnd(WM_NULL);
+
   while (m_inst_sec--)
   {
 #ifdef NSIS_CONFIG_COMPONENTPAGE
@@ -1572,7 +1679,6 @@ static BOOL CALLBACK InstProc(HWND hwndDlg, UINT uMsg, WPARAM wParam, LPARAM lPa
         i = 0;
         do {
           item.pszText = ptr;
-          item.cchTextMax = total;
           SendMessage(linsthwnd,LVM_GETITEMTEXT,i,(LPARAM)&item);
           ptr += mystrlen(ptr);
           *(WORD*)ptr = CHAR2_TO_WORD('\r','\n');
