@@ -5,8 +5,9 @@
 #include "util.h"
 #include "DialogTemplate.h"
 #include "exehead/resource.h"
+#include "version.h"
 
-extern const char *NSIS_VERSION;
+using namespace std;
 
 // Default English strings. Should match NSIS_DEFAULT_LANG
 // Do not change the first string in every item, it's the LangString
@@ -476,12 +477,12 @@ int CEXEBuild::GenerateLangTables() {
         BYTE* dlg = res_editor->GetResource(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG); \
         if (dlg) { \
           CDialogTemplate td(dlg); \
-          free(dlg); \
+          res_editor->FreeResource(dlg); \
           td.SetFont(build_font, build_font_size); \
           DWORD dwSize; \
           dlg = td.Save(dwSize); \
           res_editor->UpdateResource(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG, dlg, dwSize); \
-          res_editor->FreeResource(dlg); \
+          delete [] dlg; \
         } \
       }
 
@@ -531,13 +532,20 @@ int CEXEBuild::GenerateLangTables() {
           BYTE* dlg = res_editor->GetResource(RT_DIALOG, MAKEINTRESOURCE(id), NSIS_DEFAULT_LANG); \
           if (dlg) { \
             CDialogTemplate td(dlg,lt[i].nlf.m_uCodePage); \
-            free(dlg); \
+            res_editor->FreeResource(dlg); \
             if (font) td.SetFont(font, lt[i].nlf.m_iFontSize); \
-            if (lt[i].nlf.m_bRTL) td.ConvertToRTL(); \
+            if (lt[i].nlf.m_bRTL) { \
+              td.ConvertToRTL(); \
+              DialogItemTemplate* dir = td.GetItem(IDC_DIR); \
+              if (id == IDD_DIR && dir) { \
+                if ((dir->dwStyle & ES_CENTER) == 0) dir->dwStyle ^= ES_RIGHT; \
+                dir->dwExtStyle &= ~(WS_EX_RTLREADING | WS_EX_LEFTSCROLLBAR); \
+              } \
+            } \
             DWORD dwSize; \
             dlg = td.Save(dwSize); \
             res_editor->UpdateResource(RT_DIALOG, MAKEINTRESOURCE(id+cur_offset), NSIS_DEFAULT_LANG, dlg, dwSize); \
-            res_editor->FreeResource(dlg); \
+            delete [] dlg; \
           } \
         }
 
@@ -609,7 +617,12 @@ int CEXEBuild::GenerateLangTables() {
             {
               // No string is defined; give a warning (for user strings only)
               if (lsn[0] != '^')
-                warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
+              {
+                if (lt[i].nlf.m_bLoaded)
+                  warning("LangString \"%s\" is not set in language table of language %s", lsn, lt[i].nlf.m_szName);
+                else
+                  warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
+              }
             }
             else
             {
@@ -716,7 +729,12 @@ int CEXEBuild::GenerateLangTables() {
             {
               // No string is defined; give a warning (for user strings only)
               if (lsn[0] != '^')
-                warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
+              {
+                if (lt[i].nlf.m_bLoaded)
+                  warning("LangString \"%s\" is not set in language table of language %s", lsn, lt[i].nlf.m_szName);
+                else
+                  warning("LangString \"%s\" is not set in language table of language %d", lsn, lt[i].lang_id);
+              }
             }
             else
             {
@@ -1003,7 +1021,7 @@ LanguageTable * CEXEBuild::LoadLangFile(char *filename) {
   }
 
   // Read strings
-  for (int i = 0; i < NLF_STRINGS - 3 /* ^Font, ^FontSize and ^RTL */; i++) {
+  for (int i = 0; i < NLF_STRINGS_NO_SPECIAL; i++) {
 
     // skip virtual strings
     if (!NLFStrings[i].szDefault)
