@@ -149,7 +149,9 @@ enum {
   NLF_MEGA,
   NLF_GIGA,
 
-  NLF_FONT,
+  NLF_STRINGS_NO_SPECIAL,
+
+  NLF_FONT = NLF_STRINGS_NO_SPECIAL,
   NLF_FONTSIZE,
   NLF_RTL,
 
