@@ -3,7 +3,6 @@
 #include <stdarg.h>
 #include <time.h>
 #include <string.h>
-#include "exedata.h"
 #include "exehead/fileform.h"
 #include "util.h"
 #include "strlist.h"
@@ -15,9 +14,18 @@
 #  include <iconv.h>
 #endif
 
-#include <cassert> // for assert
+#ifdef __APPLE__
+namespace Apple { // defines struct section
+#  include <mach-o/dyld.h> // for _NSGetExecutablePath
+};
+#  include <sys/param.h> // for MAXPATHLEN
+#endif
 
-using std::string;
+#include <cassert> // for assert
+#include <algorithm>
+#include <stdexcept>
+
+using namespace std;
 
 int g_dopause=0;
 extern int g_display_errors;
@@ -39,7 +47,7 @@ void dopause(void)
 // Returns -2 if the file is an invalid bitmap
 // Returns -3 if the size doesn't match
 // Returns -4 if the bpp doesn't match
-int update_bitmap(CResourceEditor* re, WORD id, char* filename, int width/*=0*/, int height/*=0*/, int maxbpp/*=0*/) {
+int update_bitmap(CResourceEditor* re, WORD id, const char* filename, int width/*=0*/, int height/*=0*/, int maxbpp/*=0*/) {
   FILE *f = FOPEN(filename, "rb");
   if (!f) return -1;
 
@@ -52,6 +60,7 @@ int update_bitmap(CResourceEditor* re, WORD id, char* filename, int width/*=0*/,
     LONG biWidth;
     fseek(f, 18, SEEK_SET); // Seek to the width member of the header
     fread(&biWidth, sizeof(LONG), 1, f);
+    FIX_ENDIAN_INT32_INPLACE(biWidth);
     if (width != biWidth) {
       fclose(f);
       return -3;
@@ -62,6 +71,7 @@ int update_bitmap(CResourceEditor* re, WORD id, char* filename, int width/*=0*/,
     LONG biHeight;
     fseek(f, 22, SEEK_SET); // Seek to the height member of the header
     fread(&biHeight, sizeof(LONG), 1, f);
+    FIX_ENDIAN_INT32_INPLACE(biHeight);
     // Bitmap height can be negative too...
     if (height != abs(biHeight)) {
       fclose(f);
@@ -73,6 +83,7 @@ int update_bitmap(CResourceEditor* re, WORD id, char* filename, int width/*=0*/,
     WORD biBitCount;
     fseek(f, 28, SEEK_SET); // Seek to the height member of the header
     fread(&biBitCount, sizeof(WORD), 1, f);
+    FIX_ENDIAN_INT16_INPLACE(biBitCount);
     if (biBitCount > maxbpp) {
       fclose(f);
       return -4;
@@ -82,6 +93,7 @@ int update_bitmap(CResourceEditor* re, WORD id, char* filename, int width/*=0*/,
   DWORD dwSize;
   fseek(f, 2, SEEK_SET);
   fread(&dwSize, sizeof(DWORD), 1, f);
+  FIX_ENDIAN_INT32_INPLACE(dwSize);
   dwSize -= 14;
 
   unsigned char* bitmap = (unsigned char*)malloc(dwSize);
@@ -133,26 +145,41 @@ typedef struct {
 
 #define SIZEOF_RSRC_ICON_GROUP_ENTRY 14
 
-// Added by Amir Szekely 8th July 2002
-// replace_icon, must get an initialized resource editor
-// return values:
-//   0  - All OK
-//   -1 - Bad icon file
-int replace_icon(CResourceEditor* re, WORD wIconId, char* filename)
+static FILE * open_icon(const char* filename, IconGroupHeader *igh)
 {
   FILE* f = FOPEN(filename, "rb");
-  if (!f) return -1;
+  if (!f)
+    throw runtime_error("can't open file");
 
-  IconGroupHeader igh;
-  fread(&igh, sizeof(IconGroupHeader), 1, f);
+  if (!fread(igh, sizeof(IconGroupHeader), 1, f))
+    throw runtime_error("unable to read file");
 
-  if (igh.wIsIcon != 1 && igh.wReserved != 0) return -1;
+  FIX_ENDIAN_INT16_INPLACE(igh->wIsIcon);
+  FIX_ENDIAN_INT16_INPLACE(igh->wReserved);
+  FIX_ENDIAN_INT16_INPLACE(igh->wCount);
+
+  if (igh->wIsIcon != 1 || igh->wReserved != 0)
+    throw runtime_error("invalid icon file");
+
+  return f;
+}
+
+// replace_icon, must get an initialized resource editor
+void replace_icon(CResourceEditor* re, WORD wIconId, const char* filename)
+{
+  IconGroupHeader igh, *new_igh;
+  FILE *f = open_icon(filename, &igh);
 
   BYTE* rsrcIconGroup = (BYTE*)malloc(sizeof(IconGroupHeader) + igh.wCount*SIZEOF_RSRC_ICON_GROUP_ENTRY);
   if (!rsrcIconGroup) throw bad_alloc();
 
   CopyMemory(rsrcIconGroup, &igh, sizeof(IconGroupHeader));
 
+  new_igh = (IconGroupHeader *) rsrcIconGroup;
+  FIX_ENDIAN_INT16_INPLACE(new_igh->wIsIcon);
+  FIX_ENDIAN_INT16_INPLACE(new_igh->wReserved);
+  FIX_ENDIAN_INT16_INPLACE(new_igh->wCount);
+
   RsrcIconGroupEntry* ige = (RsrcIconGroupEntry*)(rsrcIconGroup + sizeof(IconGroupHeader));
 
   int i = 1;
@@ -160,36 +187,36 @@ int replace_icon(CResourceEditor* re, WORD wIconId, char* filename)
   // Delete old icons
   while (re->UpdateResource(RT_ICON, MAKEINTRESOURCE(i++), NSIS_DEFAULT_LANG, 0, 0));
 
-  int iNewIconSize = 0;
-
   for (i = 0; i < igh.wCount; i++) {
     fread(ige, sizeof(FileIconGroupEntry)-sizeof(DWORD), 1, f);
-    ige->wRsrcId = i+1;
+
+    DWORD dwRawSize = FIX_ENDIAN_INT32(ige->dwRawSize);
+
+    ige->wRsrcId = FIX_ENDIAN_INT16(i + 1);
 
     DWORD dwOffset;
     fread(&dwOffset, sizeof(DWORD), 1, f);
 
+    FIX_ENDIAN_INT32_INPLACE(dwOffset);
+
     fpos_t pos;
     fgetpos(f, &pos);
 
     if (fseek(f, dwOffset, SEEK_SET)) {
       free(rsrcIconGroup);
-      return -1;
+      throw runtime_error("corrupted icon file, too small");
     }
-    BYTE* iconData = (BYTE*)malloc(ige->dwRawSize);
+    BYTE* iconData = (BYTE*)malloc(dwRawSize);
     if (!iconData) {
       free(rsrcIconGroup);
       throw bad_alloc();
     }
-    fread(iconData, sizeof(BYTE), ige->dwRawSize, f);
-    re->UpdateResource(RT_ICON, MAKEINTRESOURCE(i+1), NSIS_DEFAULT_LANG, iconData, ige->dwRawSize);
+    fread(iconData, sizeof(BYTE), dwRawSize, f);
+    re->UpdateResource(RT_ICON, MAKEINTRESOURCE(i+1), NSIS_DEFAULT_LANG, iconData, dwRawSize);
     free(iconData);
 
     fsetpos(f, &pos);
 
-    // Every icon entry should be 8 aligned
-    iNewIconSize += ((ige->dwRawSize%8 == 0) ? ige->dwRawSize : ige->dwRawSize - (ige->dwRawSize%8) + 8);
-
     // Seems like the compiler refuses to increase the pointer by just 14.
     // If you'll replace this line by ige++ you will get unwanted results.
     ige = (RsrcIconGroupEntry*)((BYTE*)ige + SIZEOF_RSRC_ICON_GROUP_ENTRY);
@@ -200,29 +227,16 @@ int replace_icon(CResourceEditor* re, WORD wIconId, char* filename)
   re->UpdateResource(RT_GROUP_ICON, MAKEINTRESOURCE(wIconId), NSIS_DEFAULT_LANG, rsrcIconGroup, sizeof(IconGroupHeader) + igh.wCount*SIZEOF_RSRC_ICON_GROUP_ENTRY);
 
   free(rsrcIconGroup);
-
-  icondata_size = iNewIconSize;
-
-  return 0;
 }
 
-// Added by Amir Szekely 8th July 2002
 #ifdef NSIS_CONFIG_UNINSTALL_SUPPORT
 // returns the data of the uninstaller icon that should replace the installer icon data
-// return values:
-//   0  - Bad icon file
-//   Anything else - Pointer to the uninstaller icon data
-unsigned char* generate_uninstall_icon_data(char* filename)
+unsigned char* generate_uninstall_icon_data(const char* filename, size_t &size)
 {
   int i;
 
-  FILE* f = FOPEN(filename, "rb");
-  if (!f) return 0;
-
   IconGroupHeader igh;
-  if (!fread(&igh, sizeof(IconGroupHeader), 1, f)) return 0;
-
-  if (igh.wIsIcon != 1 && igh.wReserved != 0) return 0;
+  FILE *f = open_icon(filename, &igh);
 
   int iNewIconSize = 0;
   FileIconGroupEntry ige;
@@ -232,10 +246,10 @@ unsigned char* generate_uninstall_icon_data(char* filename)
   if (!offsets || !rawSizes) throw bad_alloc();
 
   for (i = 0; i < igh.wCount; i++) {
-    if (!fread(&ige, sizeof(FileIconGroupEntry), 1, f)) return 0;
+    if (!fread(&ige, sizeof(FileIconGroupEntry), 1, f)) throw runtime_error("unable to read file");
     offsets[i] = ige.dwImageOffset;
     rawSizes[i] = ige.dwRawSize;
-    iNewIconSize += ige.dwRawSize;
+    iNewIconSize += FIX_ENDIAN_INT32(ige.dwRawSize);
   }
 
   // Before each icon come two DWORDs, one for size and the other for offset (set later)
@@ -252,9 +266,9 @@ unsigned char* generate_uninstall_icon_data(char* filename)
     seeker += sizeof(DWORD);
     *(DWORD*)seeker = 0;
     seeker += sizeof(DWORD);
-    fseek(f, offsets[i], SEEK_SET);
-    fread(seeker, 1, rawSizes[i], f);
-    seeker += rawSizes[i];
+    fseek(f, FIX_ENDIAN_INT32(offsets[i]), SEEK_SET);
+    fread(seeker, 1, FIX_ENDIAN_INT32(rawSizes[i]), f);
+    seeker += FIX_ENDIAN_INT32(rawSizes[i]);
   }
 
   // This is how we know there are no more icons (size = 0)
@@ -263,7 +277,7 @@ unsigned char* generate_uninstall_icon_data(char* filename)
   free(offsets);
   free(rawSizes);
 
-  unicondata_size = iNewIconSize;
+  size = iNewIconSize;
 
   return pbUninstIcon;
 }
@@ -272,64 +286,57 @@ unsigned char* generate_uninstall_icon_data(char* filename)
 #define MY_ASSERT(x, y) if (x) {if (g_display_errors) fprintf(g_output,"\nError finding icon resources: %s -- failing!\n", y);return 0;}
 
 int find_in_dir(PRESOURCE_DIRECTORY rd, WORD id) {
-  for (int i = rd->Header.NumberOfNamedEntries; i < rd->Header.NumberOfNamedEntries + rd->Header.NumberOfIdEntries; i++) {
-    if (rd->Entries[i].Id == id) {
+  WORD i = FIX_ENDIAN_INT16(rd->Header.NumberOfNamedEntries);
+  WORD l = i + FIX_ENDIAN_INT16(rd->Header.NumberOfIdEntries);
+
+  for (; i < l; i++) {
+    if (FIX_ENDIAN_INT16(rd->Entries[i].Id) == id) {
       return i;
     }
   }
+
   return -1;
 }
 
 // Fill the array of icons for uninstall with their offsets
-// Returns 0 if failed, anything else is icon_offset.
-int generate_unicons_offsets(unsigned char* exeHeader, unsigned char* uninstIconData) {
-  int i;
+// Returns zero on failure
+int generate_unicons_offsets(unsigned char* exeHeader, size_t exeHeaderSize, unsigned char* uninstIconData) {
+  DWORD dwResourceSectionVA;
 
-  MY_ASSERT(PIMAGE_DOS_HEADER(exeHeader)->e_magic != IMAGE_DOS_SIGNATURE, "invalid dos header");
+  PIMAGE_NT_HEADERS ntHeaders = CResourceEditor::GetNTHeaders(exeHeader);
+  PRESOURCE_DIRECTORY rdRoot = CResourceEditor::GetResourceDirectory(exeHeader, exeHeaderSize, ntHeaders, &dwResourceSectionVA);
 
-  PIMAGE_NT_HEADERS ntHeaders = PIMAGE_NT_HEADERS(exeHeader + PIMAGE_DOS_HEADER(exeHeader)->e_lfanew);
+  int idx = find_in_dir(rdRoot, (WORD) (long) RT_ICON);
+  MY_ASSERT(idx < 0, "no icons found");
+  MY_IMAGE_RESOURCE_DIRECTORY_ENTRY rdEntry = rdRoot->Entries[idx];
+  FIX_ENDIAN_INT32_INPLACE(rdEntry.OffsetToData);
+  MY_ASSERT(!rdEntry.DirectoryOffset.DataIsDirectory, "bad resource directory");
 
-  MY_ASSERT(ntHeaders->Signature != IMAGE_NT_SIGNATURE, "invalid nt headers");
+  PRESOURCE_DIRECTORY rdIcons = PRESOURCE_DIRECTORY(rdEntry.DirectoryOffset.OffsetToDirectory + DWORD(rdRoot));
 
-  DWORD dwResourceSectionVA = ntHeaders->OptionalHeader.DataDirectory[IMAGE_DIRECTORY_ENTRY_RESOURCE].VirtualAddress;
-  PIMAGE_SECTION_HEADER sectionHeadersArray = IMAGE_FIRST_SECTION(ntHeaders);
+  MY_ASSERT((size_t)rdIcons - (size_t)exeHeader > exeHeaderSize, "corrupted EXE - invalid pointer");
 
-  for (i = 0; i < ntHeaders->FileHeader.NumberOfSections; i++)
-    if (dwResourceSectionVA == sectionHeadersArray[i].VirtualAddress)
-      break;
+  WORD wNumberOfEntries = FIX_ENDIAN_INT16(rdIcons->Header.NumberOfIdEntries);
 
-  MY_ASSERT(i == ntHeaders->FileHeader.NumberOfSections, "can't find resource section");
+  MY_ASSERT(wNumberOfEntries == 0, "no icons found");
 
-  PRESOURCE_DIRECTORY rdRoot = PRESOURCE_DIRECTORY(exeHeader + sectionHeadersArray[i].PointerToRawData);
+  for (WORD i = 0; i < wNumberOfEntries; i++) { // Icons dir can't have named entries
+    MY_IMAGE_RESOURCE_DIRECTORY_ENTRY icoEntry = rdIcons->Entries[i];
+    FIX_ENDIAN_INT32_INPLACE(icoEntry.OffsetToData);
 
-  int iNextSection;
-  if (i == ntHeaders->FileHeader.NumberOfSections - 1)
-    iNextSection = (int)ntHeaders->OptionalHeader.SizeOfImage;
-  else
-    iNextSection = (int)sectionHeadersArray[i+1].PointerToRawData;
-
-  MY_ASSERT((int)rdRoot - (int)exeHeader > iNextSection, "corrupted EXE - invalid pointer");
-
-  int idx = find_in_dir(rdRoot, (WORD) (int) RT_ICON);
-  MY_ASSERT(idx == -1, "no icons?!");
-  MY_ASSERT(!rdRoot->Entries[idx].DirectoryOffset.DataIsDirectory, "bad resource directory");
-
-  PRESOURCE_DIRECTORY rdIcons = PRESOURCE_DIRECTORY(rdRoot->Entries[idx].DirectoryOffset.OffsetToDirectory + DWORD(rdRoot));
-
-  MY_ASSERT((int)rdIcons - (int)exeHeader > iNextSection, "corrupted EXE - invalid pointer");
-
-  MY_ASSERT(rdIcons->Header.NumberOfIdEntries == 0, "no icons found");
-
-  for (i = 0; i < rdIcons->Header.NumberOfIdEntries; i++) { // Icons dir can't have named entries
-    MY_ASSERT(!rdIcons->Entries[i].DirectoryOffset.DataIsDirectory, "bad resource directory");
-    PRESOURCE_DIRECTORY rd = PRESOURCE_DIRECTORY(rdIcons->Entries[i].DirectoryOffset.OffsetToDirectory + DWORD(rdRoot));
+    MY_ASSERT(!icoEntry.DirectoryOffset.DataIsDirectory, "bad resource directory");
+    PRESOURCE_DIRECTORY rd = PRESOURCE_DIRECTORY(icoEntry.DirectoryOffset.OffsetToDirectory + DWORD(rdRoot));
     
-    MY_ASSERT((int)rd - (int)exeHeader > iNextSection, "corrupted EXE - invalid pointer");
-    MY_ASSERT(rd->Entries[0].DirectoryOffset.DataIsDirectory, "bad resource directory");
-    
-    PIMAGE_RESOURCE_DATA_ENTRY rde = PIMAGE_RESOURCE_DATA_ENTRY(rd->Entries[0].OffsetToData + DWORD(rdRoot));
+    MY_ASSERT((size_t)rd - (size_t)exeHeader > exeHeaderSize, "corrupted EXE - invalid pointer");
 
-    MY_ASSERT((int)rde - (int)exeHeader > iNextSection, "corrupted EXE - invalid pointer");
+    MY_IMAGE_RESOURCE_DIRECTORY_ENTRY datEntry = rd->Entries[0];
+    FIX_ENDIAN_INT32_INPLACE(datEntry.OffsetToData);
+
+    MY_ASSERT(datEntry.DirectoryOffset.DataIsDirectory, "bad resource directory");
+    
+    PIMAGE_RESOURCE_DATA_ENTRY rde = PIMAGE_RESOURCE_DATA_ENTRY(datEntry.OffsetToData + DWORD(rdRoot));
+
+    MY_ASSERT((size_t)rde - (size_t)exeHeader > exeHeaderSize, "corrupted EXE - invalid pointer");
 
     // find icon to replace
     LPBYTE seeker = uninstIconData;
@@ -341,16 +348,17 @@ int generate_unicons_offsets(unsigned char* exeHeader, unsigned char* uninstIcon
       if (!dwOffset && dwSize == rde->Size)
         break;
 
-      seeker += dwSize + sizeof(DWORD);
+      seeker += FIX_ENDIAN_INT32(dwSize) + sizeof(DWORD);
 
       // reached the end of the list and no match
       MY_ASSERT(!*seeker, "installer, uninstaller icon size mismatch - see the Icon instruction's documentation for more information");
     }
 
     // Set offset
-    *(LPDWORD) seeker = rde->OffsetToData + DWORD(rdRoot) - dwResourceSectionVA - DWORD(exeHeader);
+    DWORD dwOffset = FIX_ENDIAN_INT32(rde->OffsetToData) + DWORD(rdRoot) - dwResourceSectionVA - DWORD(exeHeader);
+    *(LPDWORD) seeker = FIX_ENDIAN_INT32(dwOffset);
 
-    MY_ASSERT(*(int*)seeker > iNextSection || *(int*)seeker < (int)rdRoot - (int)exeHeader, "invalid data offset - icon resource probably compressed");
+    MY_ASSERT(dwOffset > exeHeaderSize || dwOffset < (DWORD)rdRoot - (DWORD)exeHeader, "invalid data offset - icon resource probably compressed");
   }
 
   LPBYTE seeker = uninstIconData;
@@ -361,10 +369,10 @@ int generate_unicons_offsets(unsigned char* exeHeader, unsigned char* uninstIcon
     seeker += sizeof(DWORD);
     // offset isn't set which means we found no match for this one
     MY_ASSERT(!dwOffset, "installer, uninstaller number of icons doesn't match - see the Icon instruction's documentation for more information");
-    seeker += dwSize;
+    seeker += FIX_ENDIAN_INT32(dwSize);
   }
 
-  return PIMAGE_RESOURCE_DATA_ENTRY(PRESOURCE_DIRECTORY(rdIcons->Entries[0].DirectoryOffset.OffsetToDirectory + DWORD(rdRoot))->Entries[0].OffsetToData + DWORD(rdRoot))->OffsetToData + DWORD(rdRoot) - dwResourceSectionVA - DWORD(exeHeader);
+  return 1;
 }
 #endif // NSIS_CONFIG_UNINSTALL_SUPPORT
 
@@ -428,7 +436,7 @@ int WideCharToMultiByte(UINT CodePage, DWORD dwFlags, LPCWSTR lpWideCharStr,
   char cp[128];
   create_code_page_string(cp, sizeof(cp), CodePage);
 
-  iconv_t cd = iconv_open(cp, "UCS-2");
+  iconv_t cd = iconv_open(cp, "UCS-2LE");
   if (cd == (iconv_t) -1) {
     return 0;
   }
@@ -464,7 +472,7 @@ int MultiByteToWideChar(UINT CodePage, DWORD dwFlags, LPCSTR lpMultiByteStr,
   char cp[128];
   create_code_page_string(cp, sizeof(cp), CodePage);
 
-  iconv_t cd = iconv_open("UCS-2", cp);
+  iconv_t cd = iconv_open("UCS-2LE", cp);
   if (cd == (iconv_t) -1) {
     return 0;
   }
@@ -554,16 +562,6 @@ FILE *my_fopen(const char *path, const char *mode)
   my_convert_free(converted_path);
   return result;
 }
-
-int my_glob(const char *pattern, int flags,
-         int errfunc(const char * epath, int eerrno), glob_t *pglob)
-{
-  char *converted_pattern = my_convert(pattern);
-
-  int result = glob(converted_pattern, flags, errfunc, pglob);
-  my_convert_free(converted_pattern);
-  return result;
-}
 #endif//!_WIN32
 
 void *operator new(size_t size) {
@@ -614,32 +612,83 @@ string get_full_path(const string &path) {
 #endif//_WIN32
 }
 
-string get_dir_name(const string& path) {
-  string::size_type last_separator_pos = path.rfind(PLATFORM_PATH_SEPARATOR_C);
+string get_string_prefix(const string& str, const string& separator) {
+  const string::size_type last_separator_pos = str.rfind(separator);
   if (last_separator_pos == string::npos)
-    return path;
-  return path.substr(0, last_separator_pos);
+    return str;
+  return str.substr(0, last_separator_pos);
+}
+
+string get_string_suffix(const string& str, const string& separator) {
+  const string::size_type last_separator_pos = str.rfind(separator);
+  if (last_separator_pos == string::npos)
+    return str;
+  return str.substr(last_separator_pos + separator.size(), string::npos);
+}
+
+string get_dir_name(const string& path) {
+  return get_string_prefix(path, PLATFORM_PATH_SEPARATOR_STR);
 }
 
 string get_file_name(const string& path) {
-  string::size_type last_separator_pos = path.rfind(PLATFORM_PATH_SEPARATOR_C);
-  if (last_separator_pos == string::npos)
-    return path;
-  return path.substr(last_separator_pos + 1, string::npos);
+  return get_string_suffix(path, PLATFORM_PATH_SEPARATOR_STR);
 }
 
 string get_executable_path(const char* argv0) {
 #ifdef _WIN32
-  char temp_buf[1024];
+  char temp_buf[MAX_PATH+1];
   temp_buf[0] = '\0';
-  int rc = GetModuleFileName(NULL,temp_buf,1024);
+  int rc = GetModuleFileName(NULL,temp_buf,MAX_PATH);
   assert(rc != 0);
   return string(temp_buf);
-#else
-  return get_full_path(argv0);
+#elif __APPLE__
+  char temp_buf[MAXPATHLEN+1];
+  unsigned int buf_len = MAXPATHLEN;
+  int rc = Apple::_NSGetExecutablePath(temp_buf, &buf_len);
+  assert(rc == 0);
+  return string(temp_buf);
+#else /* Linux/BSD/POSIX/etc */
+  const char *envpath = getenv("_");
+  if( envpath != NULL ) return get_full_path( envpath );
+  else {
+    char* pathtmp;
+    char* path = NULL;
+    size_t len = 100;
+    size_t nchars;
+    while(1){
+      pathtmp = (char*)realloc(path,len+1);
+      if( pathtmp == NULL ){
+        free(path);
+        return get_full_path(argv0);
+      }
+      path = pathtmp;
+      nchars = readlink("/proc/self/exe", path, len);
+      if( nchars < 0 ){
+        free(path);
+        return get_full_path(argv0);
+      }
+      if( nchars < len ){
+        path[nchars] = '\0';
+        string result(path);
+        free(path);
+        return result;
+      }
+      len *= 2;
+    }
+  }
 #endif
 }
 
 string get_executable_dir(const char *argv0) {
   return get_dir_name(get_executable_path(argv0));
 }
+
+string remove_file_extension(const string& path) {
+  return get_string_prefix(path, ".");
+}
+
+string lowercase(const string &str) {
+  string result = str;
+  transform(str.begin(), str.end(), result.begin(), tolower);
+  return result;
+}
